@@ -62,7 +62,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
     brokerGroup.add_argument('--mqttport', type=NumberRangeArgument(1, 65535), help='Port of MQTT Broker. Default is 1883 (8883 for TLS)',
                              required=False, default=None)
     brokerGroup.add_argument('--mqttclientid', required=False, default=None, help='Id of the client. Default is a random id')
-    brokerGroup.add_argument('--prefix', help='MQTT topic prefix', type=str, required=False, default='weconnect/0')
+    brokerGroup.add_argument('--prefix', help='MQTT topic prefix (default is weconnect/0)', type=str, required=False, default='weconnect/0')
     brokerGroup.add_argument('-k', '--mqttkeepalive', required=False, type=int, default=60, help='Time between keep-alive messages')
     brokerGroup.add_argument('-mu', '--mqtt-username', type=str, dest='mqttusername', help='Username for MQTT broker', required=False)
     brokerGroup.add_argument('-mp', '--mqtt-password', type=str, dest='mqttpassword', help='Password for MQTT broker', required=False)
@@ -78,6 +78,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
     brokerGroup.add_argument('--ignore-for', dest='ignore', help='Ignore messages for first IGNORE seconds after subscribe to aviod '
                              'retained messages from the broker to make changes to the car (default is 5s) if you don\'t want this behavious set to 0',
                              type=int, required=False, default=5)
+    parser.add_argument('--list-topics', dest='listTopics', help='List new topics when created the first time', action='store_true')
 
     weConnectGroup = parser.add_argument_group('WeConnect')
     weConnectGroup.add_argument('-u', '--username', type=str, help='Username of Volkswagen id', required=False)
@@ -184,7 +185,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
 
     mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, transport=args.transport, interval=args.interval,
                                      prefix=args.prefix, ignore=args.ignore, updateCapabilities=(not args.noCapabilities),
-                                     updatePictures=args.pictures, selective=args.selective)
+                                     updatePictures=args.pictures, selective=args.selective, listNewTopics=args.listTopics)
     mqttCLient.enable_logger()
 
     if usetls:
@@ -214,7 +215,13 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
         mqttCLient.username_pw_set(username=mqttusername, password=mqttpassword)
 
     try:
-        mqttCLient.connectWeConnect(username=username, password=password, maxAgePictures=args.pictureCache)
+        while True:
+            try:
+                mqttCLient.connectWeConnect(username=username, password=password, maxAgePictures=args.pictureCache)
+                break
+            except ConnectionError as e:
+                LOG.error('Could not connect to VW-Server: %s, will retry in 10 seconds', e)
+                time.sleep(10)
 
         if args.chargingLocation is not None:
             latitude, longitude = args.chargingLocation
@@ -256,7 +263,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
 
 class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-instance-attributes
     def __init__(self, clientId=None, transport='tcp', interval=300, prefix='weconnect/0', ignore=0, updateCapabilities=True, updatePictures=True,
-                 selective=None):
+                 selective=None, listNewTopics=False):
         super().__init__(client_id=clientId, transport=transport)
         self.weConnect = None
         self.prefix = prefix
@@ -268,6 +275,11 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
         self.updateCapabilities = updateCapabilities
         self.updatePictures = updatePictures
         self.selective = selective
+        self.listNewTopics = listNewTopics
+        self.topics = []
+        self.topicsChanged = False
+        self.writeableTopics = []
+        self.writeableTopicsChanged = False
 
         self.on_connect = self.on_connect_callback
         self.on_message = self.on_message_callback
@@ -276,10 +288,44 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
 
         self.will_set(topic=f'{self.prefix}/mqtt/weconnectConnected', qos=1, retain=True, payload=False)
 
+    def addTopic(self, topic, writeable=False):
+        if topic not in self.topics:
+            self.topics.append(topic)
+            self.topics.sort()
+            self.topicsChanged = True
+
+            if writeable:
+                self.writeableTopics.append(topic)
+                self.writeableTopics.sort()
+                self.writeableTopicsChanged = True
+
+            if self.listNewTopics:
+                print(f'New topic: {topic}{" (writeable)" if writeable else ""}', flush=True)
+
+    def publishTopics(self):
+        if self.topicsChanged:
+            self.topicsChanged = False
+            topicstopic = f'{self.prefix}/mqtt/topics'
+            content = ',\n'.join(self.topics)
+            self.publish(topic=topicstopic, qos=1, retain=True, payload=content)
+            if topicstopic not in self.topics:
+                self.addTopic(topicstopic)
+
+        if self.writeableTopicsChanged:
+            self.writeableTopicsChanged = False
+            writeabletopicstopic = f'{self.prefix}/mqtt/writeableTopics'
+            content = ',\n'.join(self.writeableTopics)
+            self.publish(topic=writeabletopicstopic, qos=1, retain=True, payload=content)
+            if writeabletopicstopic not in self.topics:
+                self.addTopic(writeabletopicstopic)
+
     def disconnect(self, reasoncode=None, properties=None):
-        disconectPublish = self.publish(topic=f'{self.prefix}/mqtt/weconnectConnected', qos=1, retain=True,
-                                        payload=False)
-        disconectPublish.wait_for_publish()
+        try:
+            disconectPublish = self.publish(topic=f'{self.prefix}/mqtt/weconnectConnected', qos=1, retain=True,
+                                            payload=False)
+            disconectPublish.wait_for_publish()
+        except RuntimeError:
+            pass
         super().disconnect(reasoncode, properties)
 
     def connectWeConnect(self, username, password, maxAgePictures):
@@ -299,8 +345,11 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
             self.weConnect.update(updateCapabilities=self.updateCapabilities, updatePictures=self.updatePictures, selective=self.selective)
             self.setConnected(connected=True)
             self.setError(code=WeConnectErrors.SUCCESS)
-            self.publish(topic=f'{self.prefix}/mqtt/weconnectUpdated', qos=1, retain=True,
+            topic = f'{self.prefix}/mqtt/weconnectUpdated'
+            self.publish(topic=topic, qos=1, retain=True,
                          payload=datetime.utcnow().replace(microsecond=0, tzinfo=timezone.utc).isoformat())
+            if topic not in self.topics:
+                self.addTopic(topic)
         except errors.RetrievalError:
             self.setConnected(connected=False)
             errorMessage = f'Retrieval error during update. Will try again after configured interval of {self.interval}s'
@@ -311,12 +360,19 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
             errorMessage = f'API compatibility error during update. Will try again after configured interval of {self.interval}s'
             self.setError(code=WeConnectErrors.API_COMPATIBILITY, message=errorMessage)
             LOG.info(errorMessage)
+        self.publishTopics()
 
-    def onWeConnectEvent(self, element, flags):
+    def onWeConnectEvent(self, element, flags):  # noqa: C901
         if flags & addressable.AddressableLeaf.ObserverEvent.ENABLED:
+            topic = f'{self.prefix}{element.getGlobalAddress()}'
             if isinstance(element, addressable.ChangeableAttribute):
                 LOG.debug('Subscribe for attribute %s%s', self.prefix, element.getGlobalAddress())
-                self.subscribe(f'{self.prefix}{element.getGlobalAddress()}', qos=1)
+                self.subscribe(topic, qos=1)
+                if topic not in self.topics:
+                    self.addTopic(topic, writeable=True)
+            elif isinstance(element, addressable.AddressableAttribute):
+                if topic not in self.topics:
+                    self.addTopic(topic)
         elif flags & addressable.AddressableLeaf.ObserverEvent.VALUE_CHANGED:
             if isinstance(element.value, (str, int, float)) or element.value is None:
                 convertedValue = element.value
@@ -334,15 +390,24 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
 
     def setConnected(self, connected=True):
         if connected != self.connected:
-            self.publish(topic=f'{self.prefix}/mqtt/weconnectConnected', qos=1, retain=True, payload=connected)
+            topic = f'{self.prefix}/mqtt/weconnectConnected'
+            self.publish(topic=topic, qos=1, retain=True, payload=connected)
             self.connected = connected
+            if topic not in self.topics:
+                self.addTopic(topic)
 
     def setError(self, code=None, message=''):
         if code is None:
             code = WeConnectErrors.SUCCESS
         if code != WeConnectErrors.SUCCESS or message != '' or self.hasError is None or self.hasError:
-            self.publish(topic=f'{self.prefix}/mqtt/error/code', qos=1, retain=False, payload=code.value)
-            self.publish(topic=f'{self.prefix}/mqtt/error/message', qos=1, retain=False, payload=message)
+            topic = f'{self.prefix}/mqtt/error/code'
+            self.publish(topic=topic, qos=1, retain=False, payload=code.value)
+            if topic not in self.topics:
+                self.addTopic(topic)
+            topic = f'{self.prefix}/mqtt/error/message'
+            self.publish(topic=topic, qos=1, retain=False, payload=message)
+            if topic not in self.topics:
+                self.addTopic(topic)
         if code != WeConnectErrors.SUCCESS:
             self.hasError = True
         else:
@@ -355,12 +420,18 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
 
         if rc == 0:
             LOG.info('Connected to MQTT broker')
-            self.publish(topic=f'{self.prefix}/mqtt/weconnectForceUpdate', qos=1, payload=False)
-            self.subscribe(f'{self.prefix}/mqtt/weconnectForceUpdate', qos=2)
+            topic = f'{self.prefix}/mqtt/weconnectForceUpdate'
+            self.publish(topic=topic, qos=1, payload=False)
+            self.subscribe(topic, qos=2)
+            if topic not in self.topics:
+                self.addTopic(topic, writeable=True)
 
-            self.publish(topic=f'{self.prefix}/mqtt/weconnectUpdateInterval_s', qos=1, retain=True,
+            topic = f'{self.prefix}/mqtt/weconnectUpdateInterval_s'
+            self.publish(topic=topic, qos=1, retain=True,
                          payload=self.interval)
-            self.subscribe(f'{self.prefix}/mqtt/weconnectUpdateInterval_s', qos=1)
+            self.subscribe(topic, qos=1)
+            if topic not in self.topics:
+                self.addTopic(topic, writeable=True)
 
             self.setConnected()
 
